%   Script BSPMOTOR.M zur Erzeugung von Beispieldaten
%
%    Gleichstrommotor (ungeregelt)
%
% J. Lunze
% 13.12.1995
% fr 3. Auflage: 2.4.2001
% fr 4. Auflage: 18.9.2003   (unvernderte Parameter)
% fr 5. Auflage: 18.4.2005
% fr 6. Auflage: 30.11.2006  (Anpassung der Parameter)
%
%
clear zs ns
clear RA LA km J kL kT
echo on
%
%   Beispiel: Gleichstrommotor
%
%
%
%        Parameter:
%           RA   Widerstand im Ankerkreis
%           LA   Induktivitt der Ankerwicklung
%           km   Proportionalittsfaktor zwischen Drehzahl
%                des Ankers und Gegeninduktionsspannung
%           kL   Koeffizient der viskosen Reibung
%           kT   Proportionalittsfaktor zwischen Ankerstrom
%                und Drehmoment
%           J    Trgheitsmoment
%
%        Eingang: Spannung an Gleichstrommaschine [V]
%        Ausgang: Drehzahl [1/s]
%        Strung: Drehmoment [Nm]
%
%        Zeit in s
%
%   [zs, ns] = bertragungsfunktion Spannung -> Drehzahl
%   [zd, nd] = bertragungsfunktion Drehmoment -> Drehzahl
%   [zdappr, ndappr] = PT1-Approximation von [zd, nd]
%
%     Daten des Kleinservomotors 216 E
RA = 9;          % Ohm
LA = 0.11;       % H
km = 3 ;         % Vs/rad
kL = 0.1;        % Nms/rad
kT = km;         % Nm/A
Jm = 0.1;     % Nms^2/rad (4 * Trgheitsmoment des mechanischen Teils des Motors)


zs = [kT/2/pi];
ns = [LA*Jm (LA*kL + RA*Jm) (RA*kL + kT*km)];
MotorUF=tf(zs, ns);

zd=-1/2/pi*[LA RA];
nd=ns;
MotorStoerUF=tf(zd, nd);


%
%   Zustandsraummodell
%
%   x1 = Strom iA
%   x2 = Winkelgeschwindigkeit dphi/dt
%    u = uM Spannung am Erregerkreis
%    y = n (Drehzahl)
%
n=2;
m=1;
r=1;
A=[-RA/LA  -km/LA
    kT/Jm  -kL/Jm];
b=[1/LA
    0 ];
c=[0  1/2/pi];
d=0;
Motor=ss(A, b, c, d);

