%   Script DELTAH.M zur Herstellung von Bildern
%
%   Abhngigkeit der berschwingweite delta h
%   von Knickpunktsabstand a
%
% J. Lunze
% 13.11.1995
% fr 3. Auflage: 2.4.2001
% fr 4. Auflage: 18.9.2003
% fr 5. Auflage: 18.4.2005
% fr 7. Auflage: 28.4.2008
% fr 9. Auflage: 13.2.2012
% fr 10. Auflage: 27.1.2014
% fr 11. Auflage: 19.4.2016
% fr 12. Auflage: 7.08.2019
%
%   
clear
close all
%
%%  Abhngigkeit der berschwingweite
%   vom Knickpunktabstand
%   
figure(1)
subplot(3,1,[1,2]);
for k=1:99,
 A(k)=k*0.04;
 a=A(k);
 H(k)=100*exp(-pi*sqrt(a)/(2*sqrt(1-a/4)));
end;
plot(A, H, 'b');
axis([0 3 0 50]);
xticks([0 1 2 3]);
yticks([0 10 20 30 40 50]);
latexxlabel('$$a$$');
latexylabel('$$\Delta h$$ in \%');
rotateY;
grid('on');
latextitle('Abh\"angigkeit der \"Uberschwingweite vom Knickpunktabstand');

%%  Abhngigkeit der berschwingweite
%   vom Knickpunktabstand
%
%   
figure(2);
k=0;
H=[];
for phid=1:90
 k=k+1;
 Phid(k)=phid;
 H(k)=100*exp(-pi*cot(phid/180*pi));
end;
subplot(3,1,[1,2]);
plot(Phid, H, 'b');
hold on
plot(45, 100*exp(-pi*cot(45/180*pi)), 'bo');
plot(54, 100*exp(-pi*cot(54/180*pi)), 'bo');
axis([0 80 0 50]);
xticks([0 20 40 60 80]);
yticks([0 10 20 30 40 50]);
latexxlabel('$$\phi_{\rm d}$$ in Grad');
latexylabel('$$\Delta h$$ in \%');
rotateY;
grid('on');
latextitle('Abh\"angigkeit der \"Uberschwingweite vom der Pollage');
hold off


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%   System mit reellen Polen, das berschwingt
%
figure(3)
subplot(6,1,[1,2])
s1=-1.1;
s2=-1.5;
a1=-2.1;     %  1 < -a_1/s_1
a2=-(1+a1/s1)*s2;   %  damit die statische Verstrkung gleich eins ist
Tend=4;
Syst1=tf([a1+a2 -(a1*s2+a2*s1)],[1 -(s1+s2) s1*s2]);
[Y,Time]=step(Syst1,Tend);
plot(Time, Y, 'b');
hold on
plot([0 Tend],[1 1],'k:');
xleer;
latexylabel('$$h_{\rm w}$$');
rotateY;
axis([0 Tend -0.1 1.2]);
yticks([0 1]);
latextitle('$${\tt Deltah:}$$ \"Uberschiessen eines Systems');
hold off
%
subplot(6,1,[3,4]);
[Y,Time]=impulse(Syst1,Tend);
plot(Time, Y, 'b');
hold on
plot([0 Tend],[0 0],'k:');
%   Komponenten von g_w
Komp1=a1*exp(s1*Time);
plot(Time, Komp1, 'b--');
Komp2=a2*exp(s2*Time);
plot(Time, Komp2, 'b--');
latexxlabel('$$t$$');
latexylabel('$$g_{\rm w}$$');
rotateY;
axis([0 Tend -2.5 4.5]);
xticks([0 1 2 3 4 5]);
yticks([-2 0 2 4]);
%  Zeit tbar
tbar=log(-a1/a2)/(s2-s1);
plot(tbar, 0, 'bo');
hold off


%%    System mit I-Regler, das berschwingt
%
figure(4)
subplot(6,1,[1,2]);
s0=-2.7;
s1=0;
s2=-1.5;
G0=tf([1 -s0],[1 -(s1+s2) s1*s2]);
%rlocprintblue(G0);
kgain=1.7;
Kreis=feedback(kgain*G0, 1, -1);
[Y, Time]=step(Kreis,Tend);
plot(Time, Y, 'b');
hold on
%  zum Vergleich: System mit zwei reellen Polen
[Y,Time]=step(Syst1,Tend);
plot(Time, Y, 'b--');
plot([0 Tend],[1 1],'k:');
latexxlabel('$$t$$');
latexylabel('$$h_{\rm w}$$');
rotateY;
axis([0 Tend -0.1 1.2]);
yticks([0 1]);
xticks([0 1 2 3 4 5 6 7]);
latextitle('$${\tt Deltah:}$$ \"Uberschiessen und \"Uberschwingen');
hold off

%%     Bilder auf .EPS- und .PS-Dateien
%
Epsbildc15('deltah');
