%   Script DOMEW.M zur Herstellung von Bildern
%
%   Demonstration des Einflusses dominierender Eigenwerte
%
% J. Lunze
% 10.10.1995
% fr 2. Auflage: 12.2.1999
% fr 3. Auflage: 26.3.2001
% fr 4. Auflage: 18.9.2003
% fr 5. Auflage: 18.4.2005
% fr 6. Auflage: 4.12.2006
% fr 7. Auflage: 13.5.2008
% fr 9. Auflage: 13.2.2012
% fr 10. Auflage: 27.1.2014
% fr 12. Auflage: 01.07.2019
%
%   
clear
close all

b = [1;
     1;
     1];
c = [1 1 1];
d = 0;
T=[0:0.05:8];
Pole(1,1)=-0.8+2i;
Pole(2,1)=-0.8-2i;
for k=1:6
   e=-13+k*2;
   A = [-0.8 2   0
        -2  -0.8 0
         0   0   e];
   Pole(k+2,1)=e;
   System=ss(A, b, c, d);
   [z, n]=tfdata(System);
   System2=tf([1], n);
   [Y, X]=step(System2, T);
   ks=dcgain(System2);
   H(:,k)=Y(:,1)/ks;
end


%%     bergangsfunktion eines Systems dritter Ordnung
%      zwei Pole bei   -0.8 +j2,  -0.8 -j2
%      dritter Pol bei e = -11 ... -1
%
figure(1);
subplot(2,2,1);
plot(T, H, 'b', [0 8],[1 1],'k:');
axis([0 6 0 1.4]);
xticks([0 2 4 6]);
yticks([0 1]);
latexxlabel('$$t$$');
latexylabel('$$h$$');
rotateY;
latextitle('\"Ubergangsfunktion in Abh\"angigkeit vom dritten Pol');
latextext(1.4, 1, '-3');
latextext(0.1, 1.2, '-11');
latextext(1.9, 0.7, '-1');

%%    Lage der Pole in der komplexen Halbebene
%
subplot(2,2,2)
for i=1:length(Pole)
    plot(real(Pole(:)), imag(Pole(:)), 'bx');
end
latexxlabel('Re');
latexylabel('Im');
rotateY;
axis([-12 0 -6 6]);
axis('square');
xticks([-10 -5 0]);
yticks([-5 0 5]);
for k=1:5
   e=-13+k*2;
end;
% latextext(-10.5, 0.2, '$$\rightarrow$$');
% latextext(-8.5, 0.2, '$$\rightarrow$$');
% latextext(-6.5, 0.2, '$$\rightarrow$$');
% latextext(-4.5, 0.2, '$$\rightarrow$$');
% latextext(-2.5, 0.2, '$$\rightarrow$$');
latextext(-10, -2, '$$s_3$$');
latextext(-2, 3.8, '$$s_1$$');
latextext(-2, -3.3, '$$s_2$$');

%%     Bilder auf .EPS- und .PS-Dateien
%
Epsbildc15('domew');
