%   Script PerfregSchwinger.M
%
%   Bestimmung einer Solltrajektorie fr ein Feder-Masse-Dmpfer-System
%     Eingang: Position u
%     Ausgang: Position der Masse m_0
%
% J. Lunze
% 14.1.2010
%
% fr 8. Auflage: 25.1.2010
% fr 9. Auflage: 13.2.2012
% fr 10. Auflage: 27.1.2014
% fr 12. Auflage: 23.03.2020
%
%   
clear
close all
BspSchwinger;    %  liefert  Modell des Feder-Masse-Dmpfer-Systems
                 %  Schwinger=ss(A, b, c, d)

%
%  Transformation des Systems in E/A-Normalform
r=2;
n=5;
SS=ctrb(A, b);
SSinv=inv(SS);
sRT=SSinv(n,:);
TE=inv([c; c*A; sRT; sRT*A; sRT*A*A]);
AE=inv(TE)*A*TE;
bE=inv(TE)*b;
cE=c*TE;
EANormalform=ss(AE, bE, cE, d);
%   Nulldynamik
ANN=AE(r+1:n, r+1:n);
bq=c*A*b;
bN=AE(r+1:n,1);            % Eingang ist y(t)=w(t)
eNT=AE(r,r+1:n);
cN=-1/bq*eNT;              % Ausgabe ist uVN= -1/bq eNT*xN = uVI
dN=0;
Nulldynamik=ss(ANN, bN, cN, dN);
eIT=AE(r, 1:r);
vT=[eIT -1];


%%   Umsteuerung durch konstante Eingangsgre
%
figure(1);
ks=dcgain(Schwinger);
we=1;    % Sollposition nach Arbeitspunktwechsel
ue=we/ks;
dt=0.1;
T=0:dt:40;
Tend=40;
Tbeg=-3;
subplot(6,1,[1,2]);
plot([Tbeg 0 0 Tend], [0 0 ue ue], 'b');
xleer;
axis([Tbeg, Tend, -0.1, 1.1]);
yticks([0 1]);
latextitle('Verhalten des Feder-Masse-D\"ampfer-Systems bei konstanter Eingangsgr\"osse');
latexylabel('$$u$$  in cm');
rotateY;

subplot(6,1,[3,4]);
Y1=step(Schwinger,T)*ue;
plot(T, Y1, 'b');
hold on
plot([Tbeg 0 0],[0 0 Y1(1,1)]);
plot([Tbeg 0 0 Tend],[0 0 1 1],'b--'); % Sollwert
latexylabel('$$y$$ in cm$$\;$$');
rotateY;
axis([Tbeg Tend -0.3 1.7]);
xticks([0 10 20 30 40]);
yticks([0 1]);
latexxlabel('$$t$$ in s');
hold off;

%%   Berechnung einer Solltrajektorie
%
figure(2)
%  Umsteuerung Position der Masse m_0 von y0=0 in ye=1
Te=3;       % Endzeit in Sekunden
we=1;        %  Endwert der Position
%    Beschreibung der Trajektorie durch ein Polynom 2r-1=3. Ordnung mit aw0=0
%    Berechnung der Polynomkoeffizienten
%    aw=[aw5; aw4; aw3; aw2; aw1] als Lsung der
%    Gl. M aw= N
%    Forderung: w(0)=0; wPunkt(0)=0,
%               w(Te)=we, WPunkt(Te)=0,
aw0=0;
aw1=0;
M=[Te^3 Te^2; 3*Te^2 2*Te];
N=[we; 0];
aVektor=inv(M)*N;
aw3=aVektor(1,1);
aw2=aVektor(2,1);
WPoly=[aw3, aw2, aw1, aw0];   % Vektor der Polynomkoeffizienten von w(t)
T=[0:0.01:Te];
W=polyval(WPoly, T);                   % w(t)
WPunktPoly=[3*aw3, 2*aw2, aw1];
WPunkt=polyval(WPunktPoly, T);         % wdot(t)
WZweipunktPoly=[6*aw3, 2*aw2];
WZweipunkt=polyval(WZweipunktPoly, T); % wddot(t)
%
subplot(3,1,1);
plot(T, W, 'b')
hold on
plot(0, 0, 'bo');
plot(Te, we, 'bo');
plot([Tbeg 0 0],[0 0 W(1,1)], 'b', [Te, Te+1], [we, we], 'b');
axis([-1, Te+1 -0.2 1.2]);
xleer;
yticks([0, 1]);
latexylabel('$$w$$ in cm');
rotateY;
hold off
%
subplot(3,1,2);
plot(T, WPunkt, 'b')
hold on
plot([Tbeg 0 0],[0 0 WPunkt(1,1)], 'b',[Te, Te+1], [0 0], 'b');
latexylabel('$$\frac{\mathrm{d}w}{\mathrm{d}t}$$ in $$\frac{\rm cm}{\rm s}$$');
yticks([0, 0.5]);
axis([-1, Te+1 -0.1 0.6]);
xleer;
yticks([0 0.5]);
rotateY;
hold off
%
subplot(3,1,3);
plot(T, WZweipunkt, 'b');
hold on
LengthWZweipunkt=length(WZweipunkt);
plot([Tbeg 0 0],[0 0 WZweipunkt(1,1)], 'b',...
     [Te Te Te+1], [WZweipunkt(LengthWZweipunkt),0, 0], 'b');
latexylabel('$$\frac{\mathrm{d}^2w}{\mathrm{d}t^2}$$ in $$\frac{\rm cm}{\rm s^2}$$');
yticks([-1, 0, 1]);
axis([-1, Te+1 -1 1]);
xticks([-1 0 1 2 3 4]);
yticks([-1 0 1]);
rotateY;
latexxlabel('$$t$$ in s');
hold off


%%  Berechnung der Stellgre fr diese Solltrajektorie
%  u(t) = -1/bq eNT*xN - 1/bq (eNT  -1) x_ISoll
%       = uVN + uVI
figure(3)
%  Zeitintervall  0....Te
uVI=-1/bq*(vT(1,1)*W' + vT(1,2)*WPunkt' +vT(1,3)*WZweipunkt'); % uVI als Spaltenvektor
[uVN, T, XN]=lsim(Nulldynamik, W, T);  % Bestimmung von uVN mit XN0=0
uV=uVN+uVI;
xNe=XN(length(XN),:);    % xN(te) -  Zustand der Nulldynamik am
                         % Ende der Umsteuerung
subplot(2,1,1);
plot(T, uV, 'b');
hold on
%plot(T, uVI, 'b--');      % Anteil uVI durch Integratordynamik
plot([Tbeg 0 0], [0 0 uV(1,1)], 'b');
% Zeitintervall  Te....Tend
% hier ist y(t) = we
T2=Te:dt:Tend;
Ye=ones(size(T2))*we;                    % y ist konstant
uVI2=-1/bq*eIT(1,1)*we*ones(size(T2))';  % konstanter Anteil an u(t)
u2=lsim(Nulldynamik,Ye,T2,xNe);
u2=uVI2+u2;
plot(T2, u2, 'b');
plot([Te, Te],[uV(length(uV),1), u2(1,1)], 'b');
plot([Te Te Tend],[uV(length(uV),1) we/ks, we/ks],'b--');      %  u=konst. im Arbeitspunkt
axis([Tbeg, 25, -0.1, 1.2]);
yticks([0 1]);
latexylabel('$$u$$ in cm');
rotateY;
hold off
xleer;
%
subplot(2,1,2)
%  Zeitintervall 0...Te
[Y, T, X]=lsim(Schwinger,uV,T);
xe=X(length(X),:);    % x(te) -  Zustand der Integratordynamik am
                         % Ende der Umsteuerung
plot(T, Y, 'b', [-5 0 0],[0 0 Y(1,1)], 'b');
hold on
plot(0, 0, 'bo');
plot(Te, we, 'bo');
% Zeitintervall Te....Tend
Y2=lsim(Schwinger,u2,T2,xe);
plot(T2, Y2, 'b');
latexylabel('$$y$$ in cm$$\;$$');
rotateY;
%  im Vergleich dazu das Verhalten bei kosntanter Eingangsre
u2konst=we/ks*ones(1,length(T2));
Y2k=lsim(Schwinger,u2konst,T2,xe);
plot(T2,Y2k,'b--');
subplot(2,1,2)
plot(T2, u2konst,'b--');                   % Verhalten bei konst. Eingang
axis([Tbeg, 25, -0.1 1.3])
xticks([0 10 20 30 40]);
yticks([0 1]);
latexxlabel('$$t$$ in min');


%%
%
Epsbildc15('perfregSchwinger');
