%   Script Ptnglied.m zur Herstellung von Bildern
%
%   Abhngigkeit der berschwingweite delta h
%   von Knickpunktsabstand a
%
% J. Lunze
% fr 3. Auflage: 28.5.2001
% fr 4. Auflage: 18.9.2003
% fr 5. Auflage: 19.4.2005
% fr 6. Auflage: 4.12.2006
% fr 7. Auflage: 28.4.2008
% fr 8. Auflage: 5.1.2010
% fr 9. Auflage: 25.5.2012
% fr 10. Auflage: 27.1.2014
% fr 11. Auflage: 19.4.2016
% fr 12. Auflage: 3.07.2019
%
%   
clear
close all
T=[0:0.1:8];
PT1=tf([1], [1 1]);
PT2=PT1*PT1;
PT3=PT2*PT1;
PT4=PT3*PT1;
PT5=PT4*PT1;
%
%%  Gewichtsfunktion und bergangsfunktion eines
%   PTn-Gliedes
%
figure(1)
Y1=step(PT1, T);
Y2=step(PT2, T);
Y3=step(PT3, T);
Y4=step(PT4, T);
Y5=step(PT5, T);
%
subplot(2, 1, 1);
plot(T, Y1, 'b', T, Y2, 'b', T, Y3, 'b', T, Y4, 'b', T, Y5, 'b');
xleer;
latexylabel('$$h$$');
rotateY;
latextitle('\"Ubergangsfunktion und Gewichtsfunktion eines PT$$_n$$-Gliedes');
%axis([0 3 0 70]);
Y1=impulse(PT1, T);
Y2=impulse(PT2, T);
Y3=impulse(PT3, T);
Y4=impulse(PT4, T);
Y5=impulse(PT5, T);
latextext(0.2, 0.8, '$$n=1$$')
latextext(4.7, 0.3, '5')
axis([0 8 0 1]);
yticks([0 0.5 1]);
xleer;
%
subplot(2, 1, 2);
plot(T, Y1, 'b', T, Y2, 'b', T, Y3, 'b', T, Y4, 'b', T, Y5, 'b');
latexxlabel('$$t$$');
latexylabel('$$g$$');
rotateY;
latextext(0.5, 0.8, '$$n=1$$')
latextext(1.5, 0.5, '2')
%grid('on');
axis([0 8 0 1]);
yticks([0 0.5 1]);
xticks([0 2 4 6 8]);


%%
%   Blutalkoholkonzentration
%   
figure(2)
T1=25;                 % in Minuten
T2=35;
u=0.045*1+2*0.4*0.02;  % Alkoholmenge in Litern als Diracimpulshhe
ks=1/(17+23+5)/0.5;    % Flssigkeitsmenge
A=[-1/T1  0;
   1/T2   -1/T2];
B=[1; 0];
C=[0 ks];
Koerper=ss(A, B, C, 0);
subplot(2,1,1);
[Y3, T]=impulse(Koerper);
Y3=1000*u*Y3;
plot(T, Y3, 'b');
hold on
%
%  Berechnung des tatschlichen Verlaufs bei Abbau von 0,15 Promille pro
%  Std. nach dem Maximum bei t=29 min und y=0.8349
t1=29;
t2=120;
ymax=0.8349;
plot([t1 t2],[ymax ymax-(t2-t1)/60*0.15],'b--');
latexxlabel('$$t$$ in min');
latexylabel('\begin{minipage}{2cm} Blutalkohol in Promille\end{minipage}');
rotateY;
latextitle('Blutalkoholgehalt');
axis([0 120 0 1]);
xticks([0 50 100 150 200]);
yticks([0 0.5 1]);
hold off


%%                          Vergleichsrechnung mit konstantem Abbau
figure(3)
%                          Alkohol ins Blut
Am=[-1/T1];
Bm=[1/T1];
Cm=[ks];
AlkoholInsBlut=ss(Am, Bm, Cm, 0);
Y3m=step(AlkoholInsBlut, T);
Y3m=1000*u*Y3m;
%                          Abbau 0,15 Promille pro Stunde
deltaT=T(2)-T(1);
AbbauProZeitschritt=0.15/60*deltaT;
for k=1:length(T)
   Y3m(k)=Y3m(k)-AbbauProZeitschritt*(k-1);
end
%Y3m=max(zeros(size(Y3m)), Y3m);
plot(T, Y3m, 'b--');
latexxlabel('$$t$$ in min');
latexylabel('\begin{minipage}{2cm} Blutalkohol in Promille\end{minipage}');
rotateY;
latextitle('Blutalkoholgehalt bei konst. Abbau');
axis([0 200 0 3]);


%%     Bilder auf .EPS- und .PS-Dateien
%
Epsbildc15('ptngl');
