%   Script Schwgl.m
%
%   Verhalten des Schwingungsgliedes bei Vernderung der Pole
%
% J. Lunze
% 12.2.1999
% fr 3. Auflage: 12.2.2001
% fr 4. Auflage: 18.9.2003
% fr 5. Auflage: 18.4.2005
% fr 6. Auflage: 4.12.2006
% fr 7. Auflage: 13.5.2008
% fr 8. Auflage: 2.2.2010
% fr 9. Auflage: 13.2.2012
% fr 10. Auflage: 27.1.2014
% fr 12. Auflage: 15.07.2019
%
%   
clear
close all

%%  bergangsfunktionen bei verndertem Realteil
%   
figure(1)
omega=10;
T=[0:0.01:1.2];
for delta=-1:-1.5:-10
    z=delta*delta + omega*omega;
    Nenner=[1 -2*delta z];
    sys=tf(z, Nenner);
    Pole=roots(Nenner);
    subplot(2,3,3)
    plot(real(Pole(1,1)), imag(Pole(1,1)),'bx',  real(Pole(2,1)), imag(Pole(2,1)),'bx');
    hold on
    subplot(2,3,[1, 2]);
    [Y, T]=step(sys, T);
    plot(T, Y, 'b');
    hold on
end
%  Einfgungen in PN-Bild
subplot(2,3,3)
plot([-14 10],[0 0],'k:', [0 0],[-12 12],'k:');
axis('square');
latexxlabel('Re');
latexylabel('Im');
rotateY;
axis([-14 10 -12 12]);
xticks([-10 0 10]);
yticks([-10 0 10]);
hold off
%
subplot(2,3,[1, 2]);
plot([pi/omega pi/omega], [3 -1], 'b--')
latexxlabel('$$t$$');
latexylabel('$$h$$');
rotateY;
latextitle('\"Ubergangsfunktionen bei $$\omega=-10, \delta=-1...-10$$');
axis([0 1.2 0 2]);
xticks([0 0.2 0.4 0.6 0.8 1 1.2]);
yticks([0 1 2]);
latextext(0.35, 0.25, '$$T_{\rm m}=0,\!3$$');
hold off


%%   bergangsfunktionen bei verndertem Imaginrteil
%   
figure(2)
subplot(2,1,1);
delta=-1;
for omega=1:1.5:10
    z=delta*delta + omega*omega;
    Nenner=[1 -2*delta z];
    sys=tf(z, Nenner);
    Pole=roots(Nenner);
    subplot(2,3,3)
    plot(real(Pole(1,1)), imag(Pole(1,1)),'bx',  real(Pole(2,1)), imag(Pole(2,1)),'bx');
    hold on
    subplot(2,3,[1, 2]);
    [Y, T]=step(sys);
    plot(T, Y, 'b');
    hold on
end;
%  Einfgungen in PN-Bild
subplot(2,3,3)
plot([-14 10],[0 0],'k:', [0 0],[-12 12],'k:');
axis('square');
latexxlabel('Re');
latexylabel('Im');
rotateY;
axis([-14 10 -12 12]);
xticks([-10 0 10]);
yticks([-10 0 10]);
hold off
%
subplot(2,3,[1, 2]);
latexxlabel('$$t$$');
latexylabel('$$h$$');
rotateY;
latextitle('\"Ubergangsfunktionen bei $$\delta=-1, \omega=1...10$$');
axis([0 3 0 2]);
xticks([0 0.5 1 1.5 2 2.5 3]);
yticks([0 1 2]);
latextext(2, 1.5, '$$T_{5\%}=3$$');
hold off


%% bergangsfunktionen fr Pole auf einer Dmpfungsgeraden
%  phid = 60Grad
%   
figure(3)
subplot(2,1,1);
phid=65/180*pi;
tanphid=tan(phid);
T=[0:0.01:2];
for omega=2:1.5:10
    delta=-omega/tanphid;
    z=delta*delta + omega*omega;
    Nenner=[1 -2*delta z];
    sys=tf(z, Nenner);
    Pole=roots(Nenner);
    subplot(2,3,3)
    plot(real(Pole(1,1)), imag(Pole(1,1)),'bx',  real(Pole(2,1)), imag(Pole(2,1)),'bx');
    hold on
    subplot(2,3,[1, 2]);
    [Y, T]=step(sys);
    plot(T, Y, 'b');
    hold on
 end;
%  Einfgungen in PN-Bild
subplot(2,3,3)
plot([-14 10],[0 0],'k:', [0 0],[-12 12],'k:');
axis('square');
latexxlabel('Re');
latexylabel('Im');
rotateY;
axis([-14 10 -12 12]);
xticks([-10 0 10]);
yticks([-10 0 10]);
hold off
%
subplot(2,3,[1, 2]);
deltah=1+exp(-pi*cot(phid));
plot([0 5], [deltah, deltah], 'k:');
latexxlabel('$$t$$');
latexylabel('$$h$$');
rotateY;
latextitle('\"Ubergangsfunktionen bei $$\phi_d=65$$ Grad');
axis([0 2 0 1.3]);
xticks([0 0.5 1 1.5 2 2.5 3]);
yticks([0 0.5 1 1.2]);
latextext(0.5, 1.4, '$$\Delta h=23\%$$');
hold off

%j%   Herstellung von .ps und .eps-Bildern
%
Epsbildc15('schwgl');
