%   Script Waermet.m
%
%  Beispiel: Waermetauscher
%  Eingangsgre: w2 (Massenstrom der Khlflssigkeit)
%  Ausgangsgre: T1 (Austrittstemperatur der gekhlten Flssigkeit)
%
% J. Lunze
% fr 4. Auflage: 22.9.2003
% fr 5. Auflage: 18.4.2005
% fr 7. Auflage: 28.4.2008
% fr 8. Auflage: 4.1.2010
% fr 9. Auflage: 10.2.2012
% fr 10. Auflage: 27.1.2014
% fr 12. Auflage: 15.07.2019
%
%   
clear
close all
%                  Parameter wie in AT1
%
w1=2;               %  kg/min
w2=3;
V=2;                %  m^3
T10=85;         % Grad C
T20=20;         % Grad C
c1=1;               % cal/g/Grad
c2=c1;
m1=2;               % kg
m2=1;               % kg
A=1*0.02*pi;    % m^2  Rohr von 1 m Lnge, 2 cm Durchmesser
k=100;          % cal/min/m^2/Grad

%%  Statisches Verhalten des Wrmetauschers
%   
Nenner=w1/m1+k*A/m1/c1;
T2quer=(k*A/m2/c2/Nenner*w1/m1*T10+w2/m2*T20)/(k*A/m2/c2-(k*A/m2/c2*k*A/m1/c1)/Nenner+w2/m2)
T1quer=(k*A/m1/c1*T2quer+w1/m1*T10)/Nenner

%%     Verhalten des Wrmetauschers bei Erhhung des Durchflusses
%      des Khlmittels um 0.5
%   
figure(1)
subplot(2,1,1)
dt=0.1;
Te=7;
T=[0:dt:Te];
deltad=0;
deltaw=0.5;
sim('waermetauscher');
F=rand(size(T1))-0.5*ones(size(T1));
randvalue=0.2;  % Interval der Gleichverteilung [0, randvalue]
T1r = T1 + randvalue*F;
plot(T, T1r, 'b');
latextitle('Verhalten des W\"armetauschers');
xleer;
latexylabel('$$T_1$$  in $$^o{\rm C}\;$$');
rotateY;
axis([0 Te 50 53]);
yticks([50 51 52 53 54 55]);
%    Verhalten des Wrmetauschers bei Erhhung der Temperatur
%    der zu khlenden Flssigkeit um 5 Kelvin   
subplot(2,1,2)
deltad=5;
deltaw=0;
sim('waermetauscher');
F=rand(size(T1))-0.5*ones(size(T1));
randvalue=0.2;  % Interval der Gleichverteilung [0, randvalue]
T1r = T1 + randvalue*F;
plot(T, T1r, 'b');
latexxlabel('$$t$$ in min');
latexylabel('$$T_1$$  in $$^o{\rm C}\;$$');
rotateY;
axis([0 Te 52 55]);
yticks([50 51 52 53 54 55]);
xticks([0 1 2 3 4 5 6 7]);

%%     Bilder auf .EPS- und .PS-Dateien
%
Epsbildc15('waermet');
